from setuptools import find_packages, setup

setup(
    name='longan_sqlite',
    version='1.0.2',
    author='TakeTornado',
    author_email='cmaye@foxmail.com',
    license='MIT License',
    url='https://github.com/myemperor/longan-sqlite3',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
    ],
)
